/****************************************************************************
    Prell.c
    -------
    bungsprojekt zum Untersuchen des Tastenprellens.
    An Pin 0 von Port D wird ein mechanischer Taster zu Masse angeschlossen.
    An Port B schliessen wir ber Vorwiderstnde 4-8 LED's an, um die
    Anzahl Tastendrcke anzuzeigen. Achte darauf, dass nicht zu viel Strom
    aus dem AVR gezogen wird, also Widerstnde gengend gross whlen.
    Das Prellverhalten des Tasters muss nun mit hherer Taktfrequenz schlimmer
    sein als mit niedriger.

    Autor:    Christian Schifferle
    Datum:    Oktober 2002
    Software: AVR-GCC
    needed
****************************************************************************/

#include <io.h>	// Wir brauchen Zugriff auf die I/O's des Controllers

// Typendefinition fr ein BYTE
typedef unsigned char  BYTE;
typedef unsigned int   WORD;

// In der folgenden Variable wird der Zustand von Pin 0 an Port D
// zwischengespeichert um die Flanke zu erkennen.
// Die Variable wird im SRAM abgelegt.
BYTE   bPortD;

// In der folgenden Variable werden die Anzahl Tastendrucke gespeichert.
// Die Variable wird im SRAM abgelegt.
BYTE	nKeyPress;


int main (void)
/****************************************************************************
    Hier ist der Programmeinstiegspunkt.
*/
{

    /* Wir brauchen Port B als Ausgang. Dazu setzen wir das Datenrichtungs-
       register von Port B.
       Da wir uns den externen Pull-Up Widerstand sparen wollen verwenden
       wir den Internen von Port D.
    */
    outp (0xff,DDRB);	// Port B als Ausgang
    outp (0x00,DDRD);	// Port D als Eingang
    outp (0x01,PORTD);	// Pull-Up auf Port D, Pin 0 aktivieren

    /* Hier beginnt die Endlosschleife. Dies knnte auch als
       while (1)
       definiert werden. Der Ausdruck for (;;) ist jedoch in C
       eine bliche Schreibweise fr sowas.
    */
    bPortD = 1;	// Variable initialisieren

    for (;;) {
        /* Pin 0 von Port D abfragen, eine logische 0 heisst 'Taster gedrckt'.
            ACHTUNG: Es muss Register PIND abgefragt werden, nicht PORTD */
        if (bit_is_clear (PIND, PIND0)) {
            /* Der Taster ist gedrckt.
               Wir prfen jetzt, wie der Zustand zuvor war. */
            if (bPortD) {
                /* Schalter war vorher nicht gedrckt. */
                /* Zhler inkrementieren */
                nKeyPress++;
                /* und neuen Zhlerstand anzeigen */
                outp (nKeyPress, PORTB);
                bPortD = 0;    // Aktuellen Zustand speichern
            }
        }
        else {
            /* Taster ist nicht gedrckt */
            bPortD = 1;        // Aktuellen Zustand speichern
        }
   }
}